package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.distributed.print.business.entity.TFFlowApprove;
import com.qmth.distributed.print.business.entity.TFFlowLog;
import com.qmth.distributed.print.business.service.ActivitiService;
import com.qmth.distributed.print.business.service.TFFlowApproveService;
import com.qmth.distributed.print.business.service.TFFlowLogService;
import com.qmth.teachcloud.common.contant.SpringContextHolder;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.FlowGdykdxApproveSetupEnum;
import com.qmth.teachcloud.common.enums.FlowStatusEnum;
import com.qmth.teachcloud.common.util.ServletUtil;
import org.activiti.bpmn.model.*;
import org.activiti.bpmn.model.Process;
import org.activiti.engine.HistoryService;
import org.activiti.engine.RepositoryService;
import org.activiti.engine.RuntimeService;
import org.activiti.engine.TaskService;
import org.activiti.engine.impl.RepositoryServiceImpl;
import org.activiti.engine.impl.persistence.entity.ExecutionEntity;
import org.activiti.engine.impl.persistence.entity.ProcessDefinitionEntity;
import org.activiti.engine.impl.persistence.entity.TaskEntity;
import org.activiti.engine.repository.DeploymentBuilder;
import org.activiti.engine.runtime.ProcessInstance;
import org.activiti.engine.task.Task;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.File;
import java.io.IOException;
import java.util.*;

/**
 * @Description: activiti service impl
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/7/12
 */
@Service
public class ActivitiServiceImpl implements ActivitiService {
    private final static Logger log = LoggerFactory.getLogger(ActivitiServiceImpl.class);

    @Resource
    private RuntimeService runtimeService;

    @Resource
    private TaskService taskService;

    @Resource
    private RepositoryService repositoryService;

    @Resource
    TFFlowApproveService tfFlowApproveService;

    @Resource
    TFFlowLogService tfFlowLogService;

    @Resource
    HistoryService historyService;

    /**
     * 注册流程
     *
     * @param processFileName
     */
    @Override
    public void createDeployment(String processFileName) {
        DeploymentBuilder builder = repositoryService.createDeployment();
        builder.addClasspathResource(SystemConstant.PROCESSES + File.separator + processFileName);
        builder.deploy();
    }

    /**
     * 上传流程文件
     *
     * @param file
     */
    @Override
    public void uploadDeployment(MultipartFile file) throws IOException {
        DeploymentBuilder builder = repositoryService.createDeployment();
        builder.addInputStream(file.getOriginalFilename(), file.getInputStream());
        builder.deploy();
    }

    /**
     * 查询待办
     */
    @Override
    public void getTaskList() {
        //获取待办的一些信息，这里可以传入需要查询的用户，
        //我这里查询的所有待办
//        List<Task> tasks = taskService.createTaskQuery().taskAssignee("test123").orderByTaskCreateTime().desc().list();
        List<Task> tasks = taskService.createTaskQuery().list();
//        Task task1 = taskService.createTaskQuery().processInstanceId("22531").singleResult();
//        Task task1 = taskService.createTaskQuery()
//                .processDefinitionKey("myProcess3")
//                .taskCandidateUser("赵总")//设置候选用户
//                .singleResult();
//        Task task2 = taskService.createTaskQuery()
//                .processDefinitionKey("myProcess3")
//                .taskCandidateUser("王总")//设置候选用户
//                .singleResult();
//        Task task3 = taskService.createTaskQuery()
//                .processDefinitionKey("myProcess3")
//                .taskCandidateUser("王总1")//设置候选用户
//                .singleResult();
//        Task task4 = taskService.createTaskQuery()
//                .processDefinitionKey("myProcess3")
//                .taskCandidateUser("赵总1")//设置候选用户
//                .singleResult();
        for (Task t : tasks) {
            log.info("assignee:{},createTime:{},id:{},getProcessDefinitionId:{},name:{},processInstanceId:{},taskDefinitionKey:{},parentTaskId:{}", t.getAssignee(), t.getCreateTime(), t.getId(), t.getProcessDefinitionId(), t.getName(), t.getProcessInstanceId(), t.getTaskDefinitionKey(), t.getParentTaskId());
        }
    }

    /**
     * 根据流程key开启一个流程
     *
     * @param processKey
     * @param map
     */
    @Override
    public ProcessInstance startActivity(String processKey, Map<String, Object> map) {
        ProcessInstance processInstance = null;
        if (Objects.nonNull(map) && map.size() > 0) {
            processInstance = runtimeService.startProcessInstanceByKey(processKey, map);
        } else {
            processInstance = runtimeService.startProcessInstanceByKey(processKey);
        }
        log.info("流程id:{},processInstanceId:{},processDefinitionId:{},processDefinitionKey:{}", processInstance.getId(), processInstance.getProcessInstanceId(), processInstance.getProcessDefinitionId(), processInstance.getProcessDefinitionKey());
        ProcessInstance finalProcessInstance = processInstance;
        map.computeIfAbsent(SystemConstant.FLOW_ID, v -> finalProcessInstance.getId());
        map.computeIfAbsent(SystemConstant.FLOW_STATUS, v -> FlowStatusEnum.TO_BE_SUBMIT);
        ActivitiService activitiService = SpringContextHolder.getBean(ActivitiService.class);
        activitiService.flowStart(map);
        return processInstance;
    }

    /**
     * 根据任务id提交任务
     *
     * @param taskId
     */
    @Override
    public void complete(String taskId) {
        Map<String, Object> map = new HashMap<>();
        map.put("user", 1);
        map.put("approve", 1);
        map.put("assignee", 0);
        taskService.complete(taskId, map);
    }

    /**
     * 根据流程id直接结束流程
     *
     * @param flowId
     */
    @Override
    public void deleteProcessInstance(String flowId) {
        runtimeService.deleteProcessInstance(flowId, "结束");
    }

    /**
     * 流程启动
     *
     * @param map
     */
    @Override
    @Transactional
    public void flowStart(Map<String, Object> map) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();

        if (Objects.isNull(map.get(SystemConstant.FLOW_ID))) {
            throw ExceptionResultEnum.ERROR.exception("流程id不能为空");
        }
        Long flowId = (Long) map.get(SystemConstant.FLOW_ID);

        if (Objects.isNull(map.get(SystemConstant.FLOW_STATUS))) {
            throw ExceptionResultEnum.ERROR.exception("流程状态不能为空");
        }
        FlowStatusEnum flowStatusEnum = (FlowStatusEnum) map.get(SystemConstant.FLOW_STATUS);

        Long approveId = Objects.nonNull(map.get(SystemConstant.APPROVE_ID)) ? Long.parseLong(String.valueOf(map.get(SystemConstant.APPROVE_ID))) : null;
        tfFlowApproveService.save(new TFFlowApprove(flowId, approveId, flowStatusEnum, sysUser.getId()));
    }

    /**
     * 记录流程日志
     *
     * @param map
     */
    @Override
    @Transactional
    public void flowLog(Map<String, Object> map) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();

        if (Objects.isNull(map.get(SystemConstant.FLOW_ID))) {
            throw ExceptionResultEnum.ERROR.exception("流程id不能为空");
        }
        Long flowId = (Long) map.get(SystemConstant.FLOW_ID);

        Long taskId = null;
        if (Objects.nonNull(map.get(SystemConstant.TASK_ID))) {
            taskId = (Long) map.get(SystemConstant.TASK_ID);
        }

        if (Objects.isNull(map.get(SystemConstant.FLOW_STATUS))) {
            throw ExceptionResultEnum.ERROR.exception("流程状态不能为空");
        }
        FlowStatusEnum flowStatusEnum = (FlowStatusEnum) map.get(SystemConstant.FLOW_STATUS);

        Long approveId = Objects.nonNull(map.get(SystemConstant.APPROVE_ID)) ? Long.parseLong(String.valueOf(map.get(SystemConstant.APPROVE_ID))) : null;
        //增加流水
        ActivitiService activitiService = SpringContextHolder.getBean(ActivitiService.class);
        TFFlowLog tfFlowLog = new TFFlowLog(flowId, taskId, approveId, sysUser.getId());
    }

    /**
     * 审批任务
     *
     * @param map
     */
    @Override
    @Transactional
    public void taskApprove(Map<String, Object> map) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        if (Objects.isNull(map.get(SystemConstant.FLOW_TASK_ID))) {
            throw ExceptionResultEnum.ERROR.exception("流程任务id不能为空");
        }
        String flowTaskId = (String) map.get(SystemConstant.FLOW_TASK_ID);
        Task task = taskService.createTaskQuery().taskId(flowTaskId).singleResult();
        String processInstanceId = task.getProcessInstanceId();
        String processDefinitionId = historyService.createHistoricProcessInstanceQuery().processInstanceId(processInstanceId).singleResult().getProcessDefinitionId();
        ProcessDefinitionEntity processDefinitionEntity = (ProcessDefinitionEntity) ((RepositoryServiceImpl) repositoryService).getDeployedProcessDefinition(processDefinitionId);
        String currActivityId = task.getTaskDefinitionKey();
        BpmnModel bpmnModel = repositoryService.getBpmnModel(task.getProcessDefinitionId());
        FlowNode currFlow = (FlowNode) bpmnModel.getMainProcess().getFlowElement(currActivityId);
        QueryWrapper<TFFlowApprove> tfFlowApproveQueryWrapper = new QueryWrapper<>();
        tfFlowApproveQueryWrapper.lambda().eq(TFFlowApprove::getFlowId, processInstanceId);
        TFFlowApprove tfFlowApprove = tfFlowApproveService.getOne(tfFlowApproveQueryWrapper);
        if (currFlow instanceof UserTask) {
            UserTask userTask = (UserTask) currFlow;
            //广东医科大学流程
            if (Objects.nonNull(processDefinitionEntity) && processDefinitionEntity.getKey().contains(SystemConstant.GDYKDX_FLOW_KEY)) {
                FlowGdykdxApproveSetupEnum setupEnum = FlowGdykdxApproveSetupEnum.convertToInstance(userTask.getId());
                if (setupEnum == FlowGdykdxApproveSetupEnum.SUBMIT) {//命题提交
                    tfFlowApprove.setStatus(FlowStatusEnum.AUDITING);
                    tfFlowApprove.setSetup(FlowGdykdxApproveSetupEnum.PRIMARY_APPROVE.getSetup());
                } else if (setupEnum == FlowGdykdxApproveSetupEnum.PRIMARY_APPROVE) {//主任提交
                    if (Objects.isNull(map.get(SystemConstant.APPROVE))) {
                        throw ExceptionResultEnum.ERROR.exception("流程变量不能为空");
                    }
                    String approve = map.get(SystemConstant.APPROVE).toString();
                    if (Objects.equals(approve, "0")) {//驳回命题
                        tfFlowApprove.setStatus(FlowStatusEnum.REJECT);
                        tfFlowApprove.setSetup(FlowGdykdxApproveSetupEnum.SUBMIT.getSetup());
                    } else if (Objects.equals(approve, "1")) {//提交
                        tfFlowApprove.setStatus(FlowStatusEnum.AUDITING);
                        tfFlowApprove.setSetup(FlowGdykdxApproveSetupEnum.SECOND_APPROVE.getSetup());
                    } else {
                        throw ExceptionResultEnum.ERROR.exception("流程变量值错误");
                    }
                } else if (setupEnum == FlowGdykdxApproveSetupEnum.SECOND_APPROVE) {//院长提交
                    if (Objects.isNull(map.get(SystemConstant.APPROVE))) {
                        throw ExceptionResultEnum.ERROR.exception("流程变量不能为空");
                    }
                    String approve = map.get(SystemConstant.APPROVE).toString();
                    if (Objects.equals(approve, "0")) {//驳回命题
                        tfFlowApprove.setStatus(FlowStatusEnum.REJECT);
                        tfFlowApprove.setSetup(FlowGdykdxApproveSetupEnum.SUBMIT.getSetup());
                    } else if (Objects.equals(approve, "1")) {//驳回主任
                        tfFlowApprove.setStatus(FlowStatusEnum.REJECT);
                        tfFlowApprove.setSetup(FlowGdykdxApproveSetupEnum.PRIMARY_APPROVE.getSetup());
                    } else if (Objects.equals(approve, "2")) {//提交
                        tfFlowApprove.setStatus(FlowStatusEnum.FINISH);
                        tfFlowApprove.setSetup(FlowGdykdxApproveSetupEnum.END.getSetup());
                    } else {
                        throw ExceptionResultEnum.ERROR.exception("流程变量值错误");
                    }
                }
                tfFlowApprove.updateInfo(sysUser.getId());
            }
        }
        //当前实例的执行到哪个节点
        taskService.complete(String.valueOf(flowTaskId), map);
        tfFlowApproveService.saveOrUpdate(tfFlowApprove);
//        ActivitiService activitiService = SpringContextHolder.getBean(ActivitiService.class);
//        activitiService.flowLog(map);
    }
}
