package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.bean.params.TaskPrintParams;
import com.qmth.distributed.print.business.entity.BasicStudent;
import com.qmth.distributed.print.business.entity.ExamDetailCourse;
import com.qmth.distributed.print.business.entity.ExamStudent;
import com.qmth.distributed.print.business.entity.ExamTaskPrint;
import com.qmth.distributed.print.business.mapper.ExamTaskPrintMapper;
import com.qmth.distributed.print.business.service.*;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.util.ServletUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @Date: 2021/8/2.
 */
@Service
public class ExamTaskPrintServiceImpl extends ServiceImpl<ExamTaskPrintMapper, ExamTaskPrint> implements ExamTaskPrintService {

    @Autowired
    private BasicStudentService basicStudentService;

    @Autowired
    private ExamDetailService examDetailService;

    @Autowired
    private ExamDetailCourseService examDetailCourseService;

    @Autowired
    private ExamStudentService examStudentService;

    @Override
    public List<ExamTaskPrint> createTaskPrint(TaskPrintParams taskPrintParams) {
        Long schoolId = (Long) ServletUtil.getRequestHeaderSchoolId();
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        List<ExamTaskPrint> list = taskPrintParams.getList();
        list = list.stream().map(m -> {
            m.setPrintPlanId(taskPrintParams.getPrintPlanId());
            m.setCourseCode(taskPrintParams.getCourseCode());
            m.setCourseName(taskPrintParams.getCourseName());
            m.setPaperNumber(taskPrintParams.getPaperNumber());
            m.setExamStartTime(taskPrintParams.getExamStartTime());
            m.setExamEndTime(taskPrintParams.getExamEndTime());
            m.setSchoolId(schoolId);
            m.setOrgId(sysUser.getOrgId());
            m.setCreateId(sysUser.getId());
            return m;
        }).collect(Collectors.toList());

        this.saveBatch(list);
        return list;
    }

    @Override
    public IPage<BasicStudent> listStudentByClass(String classId, Integer pageNumber, Integer pageSize) {
        Page<BasicStudent> page = new Page<>(pageNumber, pageSize);
        IPage<BasicStudent> studentIPage = basicStudentService.listPage(page, classId);
        return studentIPage;
    }

    @Transactional
    @Override
    public void remove(Long examTaskPrintId) {
        // todo xf删除逻辑校验
        ExamTaskPrint examTaskPrint = this.getById(examTaskPrintId);
        QueryWrapper<ExamDetailCourse> examDetailCourseQueryWrapper = new QueryWrapper<>();
        examDetailCourseQueryWrapper.lambda().eq(ExamDetailCourse::getExamDetailId, examTaskPrint.getExamDetailId());
        List<ExamDetailCourse> examDetailCourses = examDetailCourseService.list(examDetailCourseQueryWrapper);
        for (ExamDetailCourse examDetailCours : examDetailCourses) {
            // 删除考务数据-考生
            UpdateWrapper<ExamStudent> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().eq(ExamStudent::getExamDetailCourseId, examDetailCours.getId());
            examStudentService.remove(updateWrapper);

            // 删除考务数据-课程
            examDetailCourseService.removeById(examDetailCours.getId());
        }

        // 删除考务数据-考点
        examDetailService.removeById(examTaskPrint.getExamDetailId());
        // 删除exam_task_print
        this.removeById(examTaskPrintId);


    }
}
